//
//  SnowGlobeViewController.m
//  SnowGlobe
//
//  Created by SJSU Student on 11/20/10.
//

#import "SnowGlobeViewController.h"

@implementation SnowGlobeViewController

NSTimer *frameTimer;	// this is the timer for animation frame updates
NSTimer *snowTimer;		// this is the timer for duration of the snowfall
BOOL snowing = NO;		// a flag to indicate animation should be running
BOOL fading = NO;		// a flag to indicate it's time for the animation to fade out

float touchStartX = 0;	// these positions used to track possible swipes
float touchStartY = 0;

// Additional setup after loading the view, namely getting the GPS set up and
// hiding the snowView view that has the snowflake animation.
- (void)viewDidLoad {
	
	// set up the GPS stuff
	locationManager = [[CLLocationManager alloc] init];
	locationManager.delegate = self;	
	locationManager.desiredAccuracy = kCLLocationAccuracyBest;
	[locationManager startUpdatingLocation];
	
	// hide the snow to start
	snowView.hidden = YES;
	
    [super viewDidLoad];
}


#pragma mark -
#pragma mark rotation methods
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;	// i.e. support all orientations
}

// actually swap and resize the background image on rotation
-(void)willAnimateRotationToInterfaceOrientation: (UIInterfaceOrientation)x
                                        duration:(NSTimeInterval)duration 
{
	UIInterfaceOrientation toOrientation = self.interfaceOrientation;
	
	// set up portrait view
	if(toOrientation == UIInterfaceOrientationPortrait || 
	   toOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		background.frame = CGRectMake(0, 0, 320, 460);
		background.image = [UIImage imageNamed:@"portrait.png"];
	}
	else {	// else set up landscape view
		background.frame = CGRectMake(0, 0, 480, 300);
		background.image = [UIImage imageNamed:@"landscape.png"];
	}
}

#pragma mark -
#pragma mark LocationManager methods
// Log errors and turn off location updates.  
- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
	if ([error code] == kCLErrorDenied) {
		NSLog(@"LocationManager Error, stopping location updates.");
		[locationManager stopUpdatingLocation];
	}
}

// When the screen is touched, display an alert with current GPS coordinate info
- (IBAction)globeTouched:(id)sender 
{
	// for fun, and to play around with CLLocation
	CLLocation *northPole = [[CLLocation alloc] initWithLatitude:90.0 longitude:0.0];
	
	// format a string with the location information of interest
	NSString *loc = [NSString stringWithFormat: 
					 @"Latitude: %fº\n"
					 "Longitude: %fº\n"
					 "Altitude: %.2f m\n"
					 "\nDistance to North Pole:\n%.2f miles",
					 locationManager.location.coordinate.latitude,
					 locationManager.location.coordinate.longitude,
					 locationManager.location.altitude,
					 [locationManager.location distanceFromLocation:northPole]*0.621371192/1000.0];
	
	// display it as an alert
	UIAlertView *alert = [[UIAlertView alloc]
						  initWithTitle:@"Current Location" 
						  message:loc
						  delegate:nil 
						  cancelButtonTitle:@"OK" 
						  otherButtonTitles:nil];
	[alert show];
	[alert release];
	[northPole release];
}

#pragma mark -
#pragma mark Snow Control methods
// Start the snow falling by unhiding the view with snow, setting flags to 
// animate it, setting the animation timers, and calling the snow method
- (IBAction)startSnow:(id)sender {
	if(!snowing) {
		snowing = YES;
		snowView.hidden = NO;
		[self snow];
		// set up the 10 second snow timer (use 8 to allow 2 second fade later)
		snowTimer = [NSTimer scheduledTimerWithTimeInterval: 8.0 
													 target:self 
												   selector:@selector(fadeSnow) 
												   userInfo:nil 
													repeats:NO];
	}
}

// this is method tells the snowView view to draw itself.  Targeted framerate
// is 30 fps.  There is also a fadeout that starts near the end of the snowfall.
- (void)snow {
	if(snowing){
		
		// check the flag and fade out if necessary
		if(fading) 
			snowView.alpha = snowView.alpha*0.97;
		
		// snowView exists as a 480 by 460 area, limit updates to just
		// the currently visible region to improve performance
		if(self.interfaceOrientation == UIInterfaceOrientationPortrait || 
		   self.interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown){
			[snowView setNeedsDisplayInRect:CGRectMake(0, 0, 320, 460)];
		}
		else {
			[snowView setNeedsDisplayInRect:CGRectMake(0, 0, 480, 300)];
		}
		// This timer specifies the targeted framerate.  This method will be 
		// called again for the next frame at the end of the timer.
		frameTimer = [NSTimer scheduledTimerWithTimeInterval: 1.0/30.0 
													  target:self 
													selector:@selector(snow) 
													userInfo:nil 
													 repeats:NO];
	}
}

// the last two seconds of the snow animation fade out slowly, this sets
// the flag for that and sets a final timer for the end of the snow.
- (void) fadeSnow {
	fading = YES;
	snowTimer = [NSTimer scheduledTimerWithTimeInterval: 2.0 
												   target:self 
												 selector:@selector(endSnow) 
												 userInfo:nil 
												  repeats:NO];
}

// Final method called in response to the shake.  It sets flags to end the 
// animation, hides the view with flakes, and resets states to properly handle 
// future shakes. 
- (void) endSnow {
	[snowView reset];
	fading = NO;
	snowing = NO;
}

#pragma mark -
#pragma mark Shake Detection
// to enable shake detection, need to add the controller to the responder chain
-(BOOL)canBecomeFirstResponder {
    return YES;
}

// this gets called when the view is added to the window, sets the controller 
// up to respond to UIEvents, like the MotionShake that we care about.
-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [self becomeFirstResponder];
}

// might want this if you ever swapped out views, but not strictly needed
- (void)viewWillDisappear:(BOOL)animated {
    [self resignFirstResponder];
    [super viewWillDisappear:animated];
}

// the actual shake detection handling: start the snow when a shake ends
- (void)motionEnded:(UIEventSubtype)motion withEvent:(UIEvent *)event
{
    if (motion == UIEventSubtypeMotionShake)
    {
		//NSLog(@"shake");
		[self startSnow:nil];
    }
}

#pragma mark -
#pragma mark touch handling code
// this detects the starts of possible touches, and sets instance state variables
// in the controller to keep track of the possible swipe
- (void) touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {

	UITouch *start = [touches anyObject];
	touchStartX = [start locationInView:snowView].x;
	touchStartY = [start locationInView:snowView].y;
}

// this called when a touch ends, decides whether it was a swipe or a tap, 
// and takes the appropriate actions
// for swipe: update the snowflake falling direction
// for tap: display GPS info
- (void) touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {

	UITouch *end = [touches anyObject];
	float touchEndX = [end locationInView:snowView].x;
	float touchEndY = [end locationInView:snowView].y;
	float deltaX = (touchEndX - touchStartX);
	float deltaY = (touchEndY - touchStartY);

	// only respond to swipes that move appreciably
	if (deltaX*deltaX + deltaY*deltaY > 200) {
		// scale magnitude of the swipe to a reasonable flake velocity and set
		snowView.xStep = deltaX/50.0;	
		snowView.yStep = deltaY/50.0;
	}
	else {	// else treat as a tap and display location info
		[self globeTouched:nil];
	}
}

#pragma mark -
#pragma mark boilerplate
- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

- (void)dealloc {
	[locationManager release];
    [super dealloc];
}

@end
